# Copyright (C) 2023 Karl Landstrom <karl@miasap.se>
#
# This file is part of gedit-convertwords.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import gi, json, os
gi.require_version("Gtk", "3.0")
gi.require_version("Gedit", "3.0")
from gi.repository import Gdk, Gtk, GObject, Gedit

conversions_per_lang = {}

class ConvertWordsAppPlugin(GObject.Object, Gedit.AppActivatable):
	__gtype_name__ = "ConvertWordsAppPlugin"

	app = GObject.Property(type=Gedit.App)

	def __init__(self):
		super().__init__()

	def do_activate(self):
		global conversions_per_lang

		dir_path = os.path.dirname(os.path.realpath(__file__))
		file_path = os.path.join(dir_path, "conversions.json")
		file = open(file_path, "r")
		if file:
			json_string = file.read()
			file.close()
			conversions_per_lang = json.loads(json_string)

	def do_deactivate(self):
		pass


class ConvertWordsWindowPlugin(GObject.Object, Gedit.WindowActivatable):
	__gtype_name__ = "ConvertWordsWindowPlugin"

	window = GObject.property(type=Gedit.Window)

	def __init__(self):
		super().__init__()

	def do_activate(self):
		pass

	def do_deactivate(self):
		pass


class ConvertWordsViewPlugin(GObject.Object, Gedit.ViewActivatable):
	__gtype_name__ = "ConvertWordsViewPlugin"

	view = GObject.property(type=Gedit.View)
	conversions = None
	key_handler_id = None
	lang_handler_id = None

	def __init__(self):
		super().__init__()

	def do_activate(self):
		doc = self.view.get_buffer()
		if doc and (self.lang_handler_id is None):
			self.lang_handler_id = doc.connect("notify::language", self.set_conversions)
			self.set_conversions(doc, None)

	def do_deactivate(self):
		if not self.key_handler_id is None:
			self.view.disconnect(self.key_handler_id)
			self.key_handler_id = None
		if not self.lang_handler_id is None:
			doc = self.view.get_buffer()
			if doc and (self.lang_handler_id is None):
				doc.disconnect(self.lang_handler_id)
				self.lang_handler_id = None

	def set_conversions(self, doc, pspec):
		lang = doc.get_language()
		if lang:
			lang_name = lang.get_name()
		else:
			lang_name = "Plain Text"
		if lang_name in conversions_per_lang:
			self.conversions = conversions_per_lang[lang_name]
			if self.key_handler_id is None:
				self.key_handler_id = self.view.connect("key-press-event", self.convert_on_completed_word)
		else:
			if not self.key_handler_id is None:
				self.view.disconnect(self.key_handler_id)
				self.key_handler_id = None

	def convert_on_completed_word(self, widget, event):
		backspace, enter = 8, 65293
		v = Gdk.keyval_to_unicode(event.keyval)  # zero for control keys and for Return on Windows
		if event.keyval == enter or not v in (0, backspace) and not chr(v).isalpha():
			doc = widget.get_buffer()
			if doc:
				# find start and end of of word
				iter = doc.get_iter_at_mark(doc.get_insert())
				end = iter.copy()
				while iter.backward_chars(1) and iter.get_char().isalpha():
					pass
				if not iter.get_char().isalpha():
					iter.forward_chars(1)
				start = iter

				# convert word if there is a translation and the word is
				# not inside a comment or string
				word = doc.get_text(start, end, False)
				if word in self.conversions:
					tags = start.get_tags()
					if len(tags) > 0:
						color = tags[0].get_property("foreground_gdk")
						highlighted = color.to_string() != "#000000000000"
					else:
						highlighted = False
					if not highlighted:
						doc.delete(start, end)
						doc.insert(start, self.conversions[word])

